@@ -157,7 +157,7 @@ struct emailinfo *nextinthread(int msgnum)
     return rp->data;
 }
 
-
+#if 0
 /*
 ** Output a menu line with hyperlinks for table display
 **
@@ -247,6 +247,8 @@ void fprint_menu(FILE *fp, mindex_t idx, char *archives, char *currentid, char *
 #endif
 }
 
+#endif
+
 /* non-tables version of fprint_menu */
 
 void fprint_menu0(FILE *fp, struct emailinfo *email, int pos)
@@ -317,6 +319,7 @@ void fprint_menu0(FILE *fp, struct emailinfo *email, int pos)
 }
 
 /*----------------------------------------------------------------------------*/
+#if 0
 
 void fprint_summary(FILE *fp, int pos, long first_d, long last_d, int num)
 {
@@ -341,6 +344,8 @@ void fprint_summary(FILE *fp, int pos, long first_d, long last_d, int num)
 #endif
 }
 
+#endif
+
 /*----------------------------------------------------------------------------*/
 
 void print_index_header_links (FILE *fp, mindex_t called_from, long startdatenum, long enddatenum, int amountmsgs, struct emailsubdir *subdir)
@@ -390,22 +395,37 @@ void print_index_header_links (FILE *fp, mindex_t called_from, long startdatenum
     }
 
     /* print the links to the other indexes */
-    if (called_from != THREAD_INDEX && show_index[dlev][THREAD_INDEX])
-      fprintf(fp, " [ <a href=\"%s\" title=\"%s\" accesskey=\"t\" rel=\"alternate\">%s</a> ]\n", 
-	      index_name[dlev][THREAD_INDEX], lang[MSG_LTITLE_BY_THREAD], lang[MSG_THREAD]);
+    if (show_index[dlev][THREAD_INDEX]) {
+      if (called_from != THREAD_INDEX)
+	fprintf(fp, " [ <a href=\"%s\" title=\"%s\" accesskey=\"t\" rel=\"alternate\">%s</a> ]\n", 
+		index_name[dlev][THREAD_INDEX], lang[MSG_LTITLE_BY_THREAD], lang[MSG_THREAD]);
+      else
+	fprintf(fp, " [ %s ]\n", lang[MSG_THREAD]);
+    }
 
-    if (called_from != AUTHOR_INDEX && show_index[dlev][AUTHOR_INDEX])
-      fprintf(fp, " [ <a href=\"%s\" title=\"%s\" accesskey=\"a\" rel=\"alternate\">%s</a> ]\n", 
-	      index_name[dlev][AUTHOR_INDEX], lang[MSG_LTITLE_BY_AUTHOR], lang[MSG_AUTHOR]);
+    if (show_index[dlev][AUTHOR_INDEX]) {
+      if (called_from != AUTHOR_INDEX)
+	fprintf(fp, " [ <a href=\"%s\" title=\"%s\" accesskey=\"a\" rel=\"alternate\">%s</a> ]\n", 
+		index_name[dlev][AUTHOR_INDEX], lang[MSG_LTITLE_BY_AUTHOR], lang[MSG_AUTHOR]);
+      else
+	fprintf(fp, " [ %s ]\n", lang[MSG_AUTHOR]);
+    }
 
-    if (called_from != DATE_INDEX && show_index[dlev][DATE_INDEX])
-      fprintf(fp, " [ <a href=\"%s\" title=\"%s\" accesskey=\"d\" rel=\"alternate\">%s</a> ]\n", 
-	      index_name[dlev][DATE_INDEX], lang[MSG_LTITLE_BY_DATE], lang[MSG_DATE]);
+    if (show_index[dlev][DATE_INDEX]) {
+      if (called_from != DATE_INDEX)
+	fprintf(fp, " [ <a href=\"%s\" title=\"%s\" accesskey=\"d\" rel=\"alternate\">%s</a> ]\n", 
+		index_name[dlev][DATE_INDEX], lang[MSG_LTITLE_BY_DATE], lang[MSG_DATE]);
+      else
+	fprintf(fp, " [ %s ]\n", lang[MSG_DATE]);
+    }
 
-
-    if (called_from != SUBJECT_INDEX && show_index[dlev][SUBJECT_INDEX])
-      fprintf(fp, " [ <a href=\"%s\" title=\"%s\" accesskey=\"s\" rel=\"alternate\">%s</a> ]\n", 
-	      index_name[dlev][SUBJECT_INDEX],  lang[MSG_LTITLE_BY_SUBJECT], lang[MSG_SUBJECT]);
+    if (show_index[dlev][SUBJECT_INDEX]) {
+      if (called_from != SUBJECT_INDEX)
+	fprintf(fp, " [ <a href=\"%s\" title=\"%s\" accesskey=\"s\" rel=\"alternate\">%s</a> ]\n", 
+		index_name[dlev][SUBJECT_INDEX],  lang[MSG_LTITLE_BY_SUBJECT], lang[MSG_SUBJECT]);
+      else
+	fprintf(fp, " [ %s ]\n", lang[MSG_SUBJECT]);
+    }
 
     if (set_attachmentsindex) {
       if (called_from != ATTACHMENT_INDEX) {
@@ -413,6 +433,8 @@ void print_index_header_links (FILE *fp, mindex_t called_from, long startdatenum
 	      index_name[dlev][ATTACHMENT_INDEX],  lang[MSG_LTITLE_BY_ATTACHMENT],
 	      lang[MSG_ATTACHMENT]);
       }
+      else
+	fprintf(fp, " [ %s ]\n", lang[MSG_ATTACHMENT]);
     }
     fprintf (fp, "</li>\n");
 
@@ -495,7 +517,7 @@ void print_index_footer_links(FILE *fp, mindex_t called_from, long enddatenum, i
     printcomment(fp, "index_footer_links", "begin");
 #endif
 
-    fprintf (fp, "<div class=\"foot\">\n");
+    fprintf (fp, "<a name=\"end\"></a><div class=\"foot\">\n");
     fprintf (fp, "<map title=\"%s\" id=\"navbarfoot\" name=\"navbarfoot\">\n", lang[MSG_NAVBAR]);
     fprintf (fp, "<ul>\n");
     
@@ -509,22 +531,37 @@ void print_index_footer_links(FILE *fp, mindex_t called_from, long enddatenum, i
 	      lang[MSG_SORT_BY]);
 
         /* print the links to the other indexes */
-    if (called_from != THREAD_INDEX && show_index[dlev][THREAD_INDEX])
-      fprintf(fp, " [ <a href=\"%s\" title=\"%s\">%s</a> ]\n", 
-	      index_name[dlev][THREAD_INDEX], lang[MSG_LTITLE_BY_THREAD], lang[MSG_THREAD]);
+    if (show_index[dlev][THREAD_INDEX]) {
+      if (called_from != THREAD_INDEX)
+	fprintf(fp, " [ <a href=\"%s\" title=\"%s\">%s</a> ]\n", 
+		index_name[dlev][THREAD_INDEX], lang[MSG_LTITLE_BY_THREAD], lang[MSG_THREAD]);
+      else
+	fprintf(fp, " [ %s ]\n", lang[MSG_THREAD]);
+    }
 
-    if (called_from != AUTHOR_INDEX && show_index[dlev][AUTHOR_INDEX])
-      fprintf(fp, " [ <a href=\"%s\" title=\"%s\">%s</a> ]\n", 
-	      index_name[dlev][AUTHOR_INDEX], lang[MSG_LTITLE_BY_AUTHOR], lang[MSG_AUTHOR]);
+    if (show_index[dlev][AUTHOR_INDEX]) {
+      if (called_from != AUTHOR_INDEX)
+	fprintf(fp, " [ <a href=\"%s\" title=\"%s\">%s</a> ]\n", 
+		index_name[dlev][AUTHOR_INDEX], lang[MSG_LTITLE_BY_AUTHOR], lang[MSG_AUTHOR]);
+      else
+	fprintf(fp, " [ %s ]\n", lang[MSG_AUTHOR]);
+    }
 
-    if (called_from != DATE_INDEX && show_index[dlev][DATE_INDEX])
-      fprintf(fp, " [ <a href=\"%s\" title=\"%s\">%s</a> ]\n", 
-	      index_name[dlev][DATE_INDEX], lang[MSG_LTITLE_BY_DATE], lang[MSG_DATE]);
+    if (show_index[dlev][DATE_INDEX]) {
+      if (called_from != DATE_INDEX)
+	fprintf(fp, " [ <a href=\"%s\" title=\"%s\">%s</a> ]\n", 
+		index_name[dlev][DATE_INDEX], lang[MSG_LTITLE_BY_DATE], lang[MSG_DATE]);
+      else
+	fprintf(fp, " [ %s ]\n", lang[MSG_DATE]);
+    }
 
-
-    if (called_from != SUBJECT_INDEX && show_index[dlev][SUBJECT_INDEX])
-      fprintf(fp, " [ <a href=\"%s\" title=\"%s\">%s</a> ]\n", 
-	      index_name[dlev][SUBJECT_INDEX], lang[MSG_LTITLE_BY_SUBJECT], lang[MSG_SUBJECT]);
+    if (show_index[dlev][SUBJECT_INDEX]) {
+      if (called_from != SUBJECT_INDEX)
+	fprintf(fp, " [ <a href=\"%s\" title=\"%s\">%s</a> ]\n", 
+		index_name[dlev][SUBJECT_INDEX], lang[MSG_LTITLE_BY_SUBJECT], lang[MSG_SUBJECT]);
+      else
+	fprintf(fp, " [ %s ]\n", lang[MSG_SUBJECT]);
+    }
 
     if (set_attachmentsindex) {
       if (called_from != ATTACHMENT_INDEX) {
@@ -532,6 +569,8 @@ void print_index_footer_links(FILE *fp, mindex_t called_from, long enddatenum, i
 	      index_name[dlev][ATTACHMENT_INDEX], lang[MSG_LTITLE_BY_ATTACHMENT], 
 	      lang[MSG_ATTACHMENT]);
       }
+      else
+	fprintf(fp, " [ %s ]\n", lang[MSG_ATTACHMENT]);
     }
     fprintf (fp, "</li>\n");
 
@@ -600,13 +639,13 @@ void print_haof_indices(FILE *fp, struct emailsubdir *subdir)
     if (show_index[dlev][DATE_INDEX])
 		fprintf(fp, "       <dateindex>%s</dateindex>\n", index_name[dlev][DATE_INDEX]);
 
-    if (show_index[dlev][AUTHOR_INDEX])
-		fprintf(fp, "       <subjectindex>%s</subjectindex>\n", index_name[dlev][AUTHOR_INDEX]);
+    if (show_index[dlev][SUBJECT_INDEX])
+		fprintf(fp, "       <subjectindex>%s</subjectindex>\n", index_name[dlev][SUBJECT_INDEX]);
 
     if (show_index[dlev][THREAD_INDEX])
 		fprintf(fp, "       <threadindex>%s</threadindex>\n", index_name[dlev][THREAD_INDEX]);
 
-    if (show_index[dlev][SUBJECT_INDEX])
+    if (show_index[dlev][AUTHOR_INDEX])
 		fprintf(fp, "       <authorindex>%s</authorindex>\n", index_name[dlev][AUTHOR_INDEX]);
 
     if (show_index[dlev][ATTACHMENT_INDEX])
@@ -721,8 +760,8 @@ void printdates(FILE *fp, struct header *hp, int year, int month, struct emailin
       subj = convchars(em->subject, em->charset);
       if(set_indextable) {
 	startline = "<tr><td>";
-	break_str = "</td><td>";
-	strcpy(date_str, getdatestr(hp->data->date));
+	break_str = "</td><td nowrap>";
+	strcpy(date_str, getdateindexdatestr(hp->data->date));
 	endline = "</td></tr>";
 	subj_tag = "";
 	subj_end_tag = "";
@@ -730,7 +769,7 @@ void printdates(FILE *fp, struct header *hp, int year, int month, struct emailin
       else {
 	char *tmp;
 	bool is_first;
-	tmp = getdatestr(hp->data->date);
+	tmp = getdateindexdatestr(hp->data->date);
 	if (strcmp (prev_date_str, tmp)) {
 	  if (*prev_date_str)  { /* close the previous date item */
 	    fprintf (fp, "</ul></li>\n");
@@ -743,16 +782,17 @@ void printdates(FILE *fp, struct header *hp, int year, int month, struct emailin
 	  fprintf (fp, "%s", date_str);
 	  strcpy (prev_date_str, tmp);
 	}
+	date_str[0] = 0;
 	startline = "<li>";
 	break_str = "&nbsp;";
 	endline = "</li>";
 	subj_tag = "";
 	subj_end_tag = "";
       }
-      fprintf(fp,"%s<a href=\"%s\">%s%s%s</a>%s<a name=\"%d\"><em>%s</em></a>%s%s\n",
+      fprintf(fp,"%s<a href=\"%s\">%s%s%s</a>%s<a name=\"%d\"><em>%s</em></a>%s%s%s\n",
 	      startline, msg_href(em, subdir_email, FALSE), 
 	      subj_tag, subj, subj_end_tag, break_str, em->msgnum, em->name,
-	      break_str, endline);
+	      break_str, date_str, endline);
       free(subj);
     }
     printdates(fp, hp->right, year, month, subdir_email, prev_date_str);
@@ -763,35 +803,42 @@ void printdates(FILE *fp, struct header *hp, int year, int month, struct emailin
 ** Pretty-prints the files with attachments in the index files.
 ** Returns the number of attachments that were printed.
 */
-int printattachments(FILE *fp, struct header *hp, struct emailinfo *subdir_email)
+int printattachments(FILE *fp, struct header *hp, struct emailinfo *subdir_email, bool *is_first)
 {
     char *subj;
     char *attdir;
     char *msgnum;
     int  nb_attach = 0;
+    static char *first_attributes = "<a  accesskey=\"j\" name=\"first\" id=\"first\"></a>";
 
-	const char *rel_path_to_top = (subdir_email ? subdir_email->subdir->rel_path_to_top : "");
+    const char *rel_path_to_top = (subdir_email ? subdir_email->subdir->rel_path_to_top : "");
 
     if (hp != NULL) {
 	struct emailinfo *em = hp->data;
-	nb_attach = printattachments(fp, hp->left, subdir_email);
+	nb_attach = printattachments(fp, hp->left, subdir_email, is_first);
 	if ((!subdir_email || subdir_email->subdir == em->subdir)
 	    && !em->is_deleted) {
 	    subj = convchars(em->subject, em->charset);
 
 	    /* See if there's a directory corresponding to this message */
-			msgnum = message_name(em);
-			trio_asprintf(&attdir, "%s%c" DIR_PREFIXER "%s", set_dir, PATH_SEPARATOR, msgnum);
+	    msgnum = message_name(em);
+	    trio_asprintf(&attdir, "%s%c" DIR_PREFIXER "%s", set_dir, PATH_SEPARATOR, msgnum);
 	    if (isdir(attdir)) {
 	        DIR *dir = opendir(attdir);
 		
 		/* consider that if there's an attachment directory, there are attachments */
 		nb_attach++;
 		if (set_indextable) {
-		  fprintf(fp, "<tr><td>%s%s</a></td><td><a name=\"%d\"><em>%s</em></a></td>" "<td>%s</td></tr>\n", msg_href(em, subdir_email, TRUE), subj, em->msgnum, em->name, getdatestr(em->date));
+		  fprintf(fp, "<tr><td>%s%s</a></td><td><a name=\"%d\"><em>%s</em></a></td>" "<td>%s</td></tr>\n", msg_href(em, subdir_email, TRUE), subj, em->msgnum, em->name, getindexdatestr(em->date));
 		}
 		else {
-		  fprintf(fp, "<li>%s<dfn>%s</dfn></a>&nbsp;" "<a name=\"%d\"><em>%s</em></a>&nbsp;<em>(%s)</em>\n", msg_href(em, subdir_email, TRUE), subj, em->msgnum, em->name, getdatestr(em->date));
+		  fprintf(fp, "<li>%s%s<dfn>%s</dfn></a>&nbsp;" 
+			  "<a name=\"%d\"><em>%s</em></a>&nbsp;<em>(%s)</em>\n", 
+			  (*is_first) ? first_attributes : "",
+			  msg_href(em, subdir_email, TRUE), subj, em->msgnum, em->name, 
+			  getindexdatestr(em->date));
+		  if (*is_first)
+		    *is_first = FALSE;
 		}
 		if (dir) {
 #ifdef HAVE_DIRENT_H
@@ -834,7 +881,7 @@ int printattachments(FILE *fp, struct header *hp, struct emailinfo *subdir_email
 	    free(attdir);
 	    free(subj);
 	}
-	nb_attach += printattachments(fp, hp->right, subdir_email);
+	nb_attach += printattachments(fp, hp->right, subdir_email, is_first);
     }
     return nb_attach;
 }
@@ -857,15 +904,19 @@ static char *ConvURLsWithHrefs(const char *line, char *mailid, char *mailsubject
     struct Push retbuf;
     char *tmpline5;
     char *tmpline6;
-    char *tmpline1 = (char *)emalloc(strlen(line) + 1);
+    char *tmpline1;
 
+    /* only do the convertion if a complete href is found */
+    p = strcasestr(c, "</a>");
+    if (!p)
+      return NULL;
+
+    tmpline1 = (char *)emalloc(strlen(line) + 1);
     strncpy(tmpline1, line, c - line);	/* AUDIT biege: who gurantees that c-line doesnt become smaller 0? IOF? */
     tmpline1[c - line] = 0;
     INIT_PUSH(retbuf);
-    p = strcasestr(c, "</a>");
-	if (!p)
-		return NULL;
-	/* complete href found; run ConvURLsString on text outside it */
+
+    /* complete href found; run ConvURLsString on text outside it */
     tmpline5 = ConvURLsString(tmpline1, mailid, mailsubject, charset);
 	if (!tmpline5)
 		return NULL;
@@ -979,6 +1030,13 @@ char *ConvURLsString(char *line, char *mailid, char *mailsubject, char *charset)
 
     parsed = parseurl(line, charset);
 
+    /* as at this point we don't know how to separate the href convertions from mailto
+       ones in the same line, we keep frmo doing the mailto convertion if we find a
+       complete href */
+    if (parsed && strcasestr(parsed, "</a>"))
+      return parsed;
+
+    /* we didn't find any previous href convertion, we try to do a mailto: convertion */
     if (use_mailcommand) {
 	/* Exclude headers that are not mail type headers */
 
@@ -1018,6 +1076,9 @@ void printheaders (FILE *fp, struct emailinfo *email)
     
     if (set_show_headers) {
       while (bp != NULL && bp->header) {
+        if ((bp->line)[0] == '\n') {   /* don't try to convert newline */
+          break;
+        }
 
 	if (sscanf(bp->line, "%127[^:]", head) == 1 && showheader(head)) {
 	  /* this is a header we want to show */
@@ -1339,7 +1400,7 @@ void print_headers(FILE *fp, struct emailinfo *email, int in_thread_file)
 	free(ptr);
     }
     else {
-      fprintf(fp, "%s ^lt;<em>%s</em>&gt;", email->name, 
+      fprintf(fp, "%s &lt;<em>%s</em>&gt;", email->name, 
 	      (strcmp(email->emailaddr, "(no email)") != 0) ? email->emailaddr : "no email");
     }
   }
@@ -1349,7 +1410,7 @@ void print_headers(FILE *fp, struct emailinfo *email, int in_thread_file)
   if (in_thread_file)
     fprintf(fp, "<span id=\"subject\"><dfn>%s</dfn>: %s</span><br />\n", lang[MSG_SUBJECT], email->subject);
   /* date */
-  fprintf(fp, "<span id=\"date\"><dfn>%s</dfn>: %s</span><br />\n", lang[MSG_CDATE], getdatestr(email->date));
+  fprintf(fp, "<span id=\"date\"><dfn>%s</dfn>: %s</span><br />\n", lang[MSG_CDATE], email->datestr);
 
   printheaders (fp, email);
 
@@ -1371,6 +1432,44 @@ static char *href01(struct emailinfo *email, struct emailinfo *email2, int in_th
 	  return msg_href(email2, email, generate_markup);
 }
 
+static void
+print_replies(FILE *fp, struct emailinfo *email, int num, int in_thread_file)
+{
+    struct reply *rp;
+    struct emailinfo *email2;
+    char *ptr;
+    bool list_started = FALSE;
+#ifdef FASTREPLYCODE
+    for (rp = email->replylist; rp != NULL; rp = rp->next) {
+	if (hashnumlookup(rp->msgnum, &email2)) {
+#else
+    for (rp = replylist; rp != NULL; rp = rp->next) {
+        if (rp->frommsgnum == num && hashnumlookup(rp->msgnum, &email2)) {
+#endif
+	    char *del_msg = (email2->is_deleted ? lang[MSG_DEL_SHORT] : "");
+	    if (!list_started) {
+	        list_started = TRUE;
+		fprintf (fp, "<li><a name=\"replies\" id=\"replies\"></a>\n");
+	    }
+	    else
+	        fprintf (fp, "<li>");
+
+	    if (rp->maybereply)
+		fprintf(fp, "<dfn>%s</dfn>:", lang[MSG_MAYBE_REPLY]);
+	    else
+	        fprintf(fp, "<dfn>%s</dfn>:", lang[MSG_REPLY]);
+	    fprintf(fp, "%s <a href=\"%s\" title=\"%s\">", del_msg, 
+		    href01(email, email2, in_thread_file, FALSE),
+		    lang[MSG_LTITLE_REPLIES]);
+	    ptr = convchars(email2->subject, email2->charset);
+	    fprintf(fp, "%s: \"%s\"</a></li>\n", email2->name, ptr);
+	    if (ptr)
+		free(ptr);
+	}
+    }
+    printcomment(fp, "lreply", "end");
+}
+
 int print_links_up(FILE *fp, struct emailinfo *email, int pos, int in_thread_file)
 {
 	int num = email->msgnum;
@@ -1492,7 +1591,11 @@ int print_links_up(FILE *fp, struct emailinfo *email, int pos, int in_thread_fil
 	     bottom reply section.
 	     */
 	
-	    if (set_showreplies) {
+	    if (set_show_msg_links == 3) {
+	        /* print_replies(fp, email, num, in_thread_file); */
+		is_reply = print_links(fp, email, pos, in_thread_file);
+	    }
+	    else if (set_showreplies) {
 #ifdef FASTREPLYCODE
 	      for (rp = email->replylist; rp != NULL; rp = rp->next) {
 		if (hashnumlookup(rp->msgnum, &email2)) {
@@ -1521,7 +1624,6 @@ int print_links(FILE *fp, struct emailinfo *email, int pos, int in_thread_file)
 	struct emailinfo *email2;
 	struct emailinfo *email_next_in_thread = nextinthread(email->msgnum);
 	char *ptr;
-	struct reply *rp;
 	int is_reply = 0;
 	int loc_cmp = (pos == PAGE_BOTTOM ? 3 : 4);
 
@@ -1617,46 +1719,16 @@ int print_links(FILE *fp, struct emailinfo *email, int pos, int in_thread_file)
 	/*
 	 * Does this message have replies? If so, print them all!
 	 */
-	
+
 	if (set_showreplies) {
-	  bool list_started = FALSE;
-#ifdef FASTREPLYCODE
-	  for (rp = email->replylist; rp != NULL; rp = rp->next) {
-                    if (hashnumlookup(rp->msgnum, &email2)) {
-#else
-		      for (rp = replylist; rp != NULL; rp = rp->next) {
-			if (rp->frommsgnum == num && hashnumlookup(rp->msgnum, &email2)) {
-#endif
-			  char *del_msg = (email2->is_deleted ? lang[MSG_DEL_SHORT] : "");
-			  if (!list_started) {
-			    list_started = TRUE;
-			    fprintf (fp, "<li><a name=\"replies\" id=\"replies\"></a>\n");
-			  }
-			  else
-			    fprintf (fp, "<li>");
+	    print_replies(fp, email, num, in_thread_file);
 
-			  if (rp->maybereply)
-			    fprintf(fp, "<dfn>%s</dfn>:", lang[MSG_MAYBE_REPLY]);
-			  else
-			    fprintf(fp, "<dfn>%s</dfn>:", lang[MSG_REPLY]);
-			  fprintf(fp, "%s <a href=\"%s\" title=\"%s\">", del_msg, 
-				  href01(email, email2, in_thread_file, FALSE),
-				  lang[MSG_LTITLE_REPLIES]);
-			  ptr = convchars(email2->subject, email2->charset);
-			  fprintf(fp, "%s: \"%s\"</a></li>\n", 
-				  email2->name, ptr);
-			  if (ptr)
-			    free(ptr);
-			}
-		      }
-		      printcomment(fp, "lreply", "end");
-		    }
-
-		    /* close the list */
-		    fprintf(fp, "</ul>\n");
-	  }
-	  return is_reply;
+	    /* close the list */
+	    fprintf(fp, "</ul>\n");
 	}
+	}
+	return is_reply;
+}
 
 static int has_new_replies(struct emailinfo *email)
 {
@@ -1703,6 +1775,17 @@ static GDBM_FILE gdbm_init()
 
 	    unlink(indexname);
 	}
+	else {
+	    datum key;
+	    datum content;
+	    char buf[512];
+	    key.dptr = "delete_level";
+	    key.dsize = strlen(key.dptr);
+	    sprintf(buf, "%d", set_delete_level);
+	    content.dsize = strlen(buf) + 1;
+	    content.dptr = buf; /* the value is in this string */
+	    gdbm_store(gp, key, content, GDBM_REPLACE);
+	}
     }
     return gp;
 }
@@ -1724,17 +1807,22 @@ void update_deletions(int num_old)
 	if (num >= num_old)
 	    continue;		/* new message - already done */
 	if (hashnumlookup(num, &ep)) {
-	    char *filename = articlehtmlfilename(ep);
-	    if (set_delete_level != DELETE_REMOVES_FILES) {
-		struct body *bp = ep->bodylist;
-		if (bp == NULL)
-		    parse_old_html(num, ep, 1, 0, NULL);
-		writearticles(num, num + 1);
+	    char *filename;
+	    if (ep->deletion_completed == set_delete_level) /* done already? */
+	        continue;
+	    if (set_delete_level != DELETE_LEAVES_TEXT) {
+	        filename = articlehtmlfilename(ep);
+		if (set_delete_level != DELETE_REMOVES_FILES) {
+		    struct body *bp = ep->bodylist;
+		    if (bp == NULL)
+		        parse_old_html(num, ep, 1, 0, NULL, 0);
+		    writearticles(num, num + 1);
+		}
+		else if (isfile(filename)) {
+		    unlink(filename);
+		}
+		free(filename);
 	    }
-	    else if (isfile(filename)) {
-		unlink(filename);
-	    }
-	    free(filename);
 #ifdef FASTREPLYCODE
 	    for (rp = ep->replylist; rp != NULL; rp = rp->next) {
 #else
@@ -1743,7 +1831,7 @@ void update_deletions(int num_old)
 	        int rnum = rp->data->msgnum;
 		if (rnum < num_old) {
 		    if (!rp->data->bodylist || !rp->data->bodylist->line[0])
-		        parse_old_html(rnum, rp->data, TRUE, FALSE, NULL);
+		        parse_old_html(rnum, rp->data, TRUE, FALSE, NULL, 0);
 			writearticles(rnum, rnum + 1);	/* update MSG_IN_REPLY_TO line */
 		}
 	    }
@@ -1844,7 +1932,10 @@ void writearticles(int startnum, int maxnum)
 	  if ((fp = fopen(filename, "w")) == NULL) { /* AUDIT biege:where? */
 	        snprintf(errmsg, sizeof(errmsg), "%s \"%s\".", lang[MSG_COULD_NOT_WRITE], filename);
 		progerr(errmsg);
-	    }
+	  }
+	  if (set_report_new_file) {
+	      printf("%s\n", filename);
+	  }
 	}
 
 	email_next_in_thread = nextinthread(email->msgnum);
@@ -1918,7 +2009,7 @@ void writearticles(int startnum, int maxnum)
 	print_headers(fp, email, FALSE);
 	printbody(fp, email, maybe_reply, is_reply);
 	fprintf (fp, "<span id=\"received\"><dfn>%s</dfn> %s</span>\n", 
-		 lang[MSG_RECEIVED_ON], email->fromdatestr);
+		 lang[MSG_RECEIVED_ON],  getdatestr(email->fromdate));
 	fprintf (fp, "</div>\n");
 	printcomment(fp, "body", "end");
 
@@ -2066,7 +2157,7 @@ void writedates(int amountmsgs, struct emailinfo *email)
      * Print out the actual message index lists. Here's the beef.
      */
     if (set_indextable)
-	fprintf(fp, "<div class=\"center\">\n<table width=\"80%%\">\n<tr><td><strong>%s</strong></td><td><strong>%s</strong></td><td><strong>%s</strong></td></tr>\n", lang[MSG_CSUBJECT], lang[MSG_CAUTHOR], lang[MSG_CDATE]);
+	fprintf(fp, "<div class=\"center\">\n<table>\n<tr><td><strong>%s</strong></td><td><strong>%s</strong></td><td><strong>%s</strong></td></tr>\n", lang[MSG_CSUBJECT], lang[MSG_CAUTHOR], lang[MSG_CDATE]);
     else {
         fprintf (fp, "<div class=\"messages-list\">\n");
 	fprintf(fp, "<ul>\n");
@@ -2121,6 +2212,7 @@ void writeattachments(int amountmsgs, struct emailinfo *email)
     char *attname = index_name[email && email->subdir != NULL][ATTACHMENT_INDEX];
     time_t start_date_num = email && email->subdir ? email->subdir->first_email->date : firstdatenum;
     time_t end_date_num = email && email->subdir ? email->subdir->last_email->date : lastdatenum;
+    bool is_first = TRUE; /* used to print the anchor to the first element (WAI) */
 
     filename = htmlfilename(attname, email, "");
 
@@ -2154,14 +2246,14 @@ void writeattachments(int amountmsgs, struct emailinfo *email)
      */
 
     if (set_indextable) {
-		fprintf(fp, "<div class=\"center\">\n<table width=\"80%%\">\n<tr><td><strong>%s</strong></td><td><strong>%s</strong></td><td><strong>%s</strong></td></tr>\n", lang[MSG_CSUBJECT], lang[MSG_CAUTHOR], lang[MSG_CDATE]);
-	printattachments(fp, datelist, email);
+	fprintf(fp, "<div class=\"center\">\n<table>\n<tr><td><strong>%s</strong></td><td><strong>%s</strong></td><td><strong>%s</strong></td></tr>\n", lang[MSG_CSUBJECT], lang[MSG_CAUTHOR], lang[MSG_CDATE]);
+	printattachments(fp, datelist, email, &is_first);
 	fprintf(fp, "</table>\n</div>\n");
     }
     else {
         fprintf (fp, "<div class=\"messages-list\">\n");
 	fprintf(fp, "<ul>\n");
-	if (printattachments(fp, datelist, email) == 0)
+	if (printattachments(fp, datelist, email, &is_first) == 0)
 	   fprintf(fp, "<li style=\"display: none\">Nothing received yet!</li>\n");
 	fprintf(fp, "</ul>\n");
 	fprintf(fp, "</div>\n");
@@ -2177,7 +2269,7 @@ void writeattachments(int amountmsgs, struct emailinfo *email)
     /* 
      * Print the index page footer.
      */
-	printfooter(fp, ihtmlfooterfile, set_label, set_dir, lang[MSG_BY_DATE], attname, TRUE);
+    printfooter(fp, ihtmlfooterfile, set_label, set_dir, lang[MSG_BY_DATE], attname, TRUE);
 
     fclose(fp);
 
@@ -2284,6 +2376,7 @@ void printsubjects(FILE *fp, struct header *hp, char **oldsubject,
   const char *break_str;
   const char *endline;
   static char date_str[DATESTRLEN+11]; /* made static for smaller stack */
+  static char *first_attributes = "<a  accesskey=\"j\" name=\"first\" id=\"first\"></a>";
 
   if (hp != NULL) {
     printsubjects(fp, hp->left, oldsubject, year, month, subdir_email);
@@ -2300,22 +2393,29 @@ void printsubjects(FILE *fp, struct header *hp, char **oldsubject,
 			subj);
 	    }
 	    else {
-	        if (*oldsubject && *oldsubject[0] != '\0')  /* close the previous open list */
+	      bool is_first;
+	        if (*oldsubject && *oldsubject[0] != '\0')  { /* close the previous open list */
 		  fprintf(fp, "</ul></li>\n");
-		fprintf(fp, "<li><dfn>%s</dfn>\n", subj);
+		  is_first = FALSE;
+		} 
+		else
+		  is_first = TRUE;
+
+		fprintf(fp, "<li>%s<dfn>%s</dfn>\n", 
+			(is_first) ? first_attributes : "", subj);
 		fprintf(fp, "<ul>\n");
 	    }
 	}
 	if(set_indextable) {
-	    startline = "<tr><td>&nbsp;</td><td>";
-	    break_str = "</td><td>";
-	    strcpy(date_str, getdatestr(hp->data->date));
+	    startline = "<tr><td>&nbsp;</td><td nowrap>";
+	    break_str = "</td><td nowrap>";
+	    strcpy(date_str, getindexdatestr(hp->data->date));
 	    endline = "</td></tr>";
 	}
 	else {
 	    startline = "<li>";
 	    break_str = "";
-	    sprintf(date_str, "<em>(%s)</em>", getdatestr(hp->data->date));
+	    sprintf(date_str, "<em>(%s)</em>", getindexdatestr(hp->data->date));
 	    endline = "</li>";
 	}
 	fprintf(fp,
@@ -2421,6 +2521,7 @@ void printauthors(FILE *fp, struct header *hp, char **oldname,
   const char *break_str;
   const char *endline;
   static char date_str[DATESTRLEN+11]; /* made static for smaller stack */
+  static char *first_attributes = "<a  accesskey=\"j\" name=\"first\" id=\"first\"></a>";
 
   if (hp != NULL) {
     printauthors(fp, hp->left, oldname, year, month, subdir_email);
@@ -2434,26 +2535,35 @@ void printauthors(FILE *fp, struct header *hp, char **oldname,
       if (strcasecmp(hp->data->name, *oldname)) {
 
 	if(set_indextable)
-		fprintf(fp,
-			"<tr><td colspan=\"3\"><strong>%s</strong></td></tr>",
-			hp->data->name);
+	  fprintf(fp,
+		  "<tr><td colspan=\"3\"><strong>%s</strong></td></tr>",
+		  hp->data->name);
 	else {
-		if (*oldname && *oldname[0] != '\0') /* close the previous open list */
-		  fprintf(fp, "</ul></li>\n");
-		fprintf(fp, "<li><dfn>%s</dfn>\n", hp->data->name);
-		fprintf(fp, "<ul>\n");
+	  bool is_first;
+
+	  if (*oldname && *oldname[0] != '\0') { /* close the previous open list */
+	    fprintf(fp, "</ul></li>\n");
+	    is_first = FALSE;
+	  }
+	  else
+	    is_first = TRUE;
+
+	  fprintf(fp, "<li>%s<dfn>%s</dfn>\n", 
+		  (is_first) ? first_attributes : "",
+		  hp->data->name);
+	  fprintf(fp, "<ul>\n");
 	}
       }
       if(set_indextable) {
 	startline = "<tr><td>&nbsp;</td><td>";
-	break_str = "</td><td>";
-	strcpy(date_str, getdatestr(hp->data->date));
+	break_str = "</td><td nowrap>";
+	strcpy(date_str, getindexdatestr(hp->data->date));
 	endline = "</td></tr>";
       }
       else {
 	startline = "<li>";
 	break_str = "&nbsp;";
-	sprintf(date_str, "<em>(%s)</em>", getdatestr(hp->data->date));
+	sprintf(date_str, "<em>(%s)</em>", getindexdatestr(hp->data->date));
 	endline = "</li>";
       }
       fprintf(fp,"%s%s%s</a>%s<a name=\"%d\">%s</a>%s\n",
@@ -2586,7 +2696,6 @@ void writehaof(int amountmsgs, struct emailinfo *email)
     FILE *fp;
 
     filename = haofname(email);
-    printf("haofname %s\n", filename);
 
     if (isfile(filename))
 	newfile = 0;
@@ -2812,15 +2921,19 @@ void write_toplevel_indices(int amountmsgs)
     bool first = TRUE;
     struct emailsubdir *sd;
     char *subject = lang[MSG_FOLDERS_INDEX];
-    static char period_name[DATESTRLEN*2 + 1];
+    static char verbose_period_name[DATESTRLEN*2 + 1];
+    static char abbr_period_name[DATESTRLEN*2 + 1];
     char *end_date;
     char *index_title;
     char *filename;
     char *saved_set_dateformat;
+    char *abbr_dateformat = "%e %b %Y";
+    char *verbose_dateformat = "%A, %e %B %Y";
+
     FILE *fp;
 
     filename = htmlfilename(index_name[0][FOLDERS_INDEX], NULL, "");
-    if (isfile(filename))
+    if (isfile(filename)) 
 	newfile = 0;
     else
 	newfile = 1;
@@ -2856,7 +2969,6 @@ void write_toplevel_indices(int amountmsgs)
 	while (sd->next_subdir)
 	    sd = sd->next_subdir;
     saved_set_dateformat = set_dateformat;
-    set_dateformat = "%d %b %Y";
     for (; sd != NULL; sd = set_reverse_folders ? sd->prior_subdir : sd->next_subdir) {
 	int started_line = 0;
 	if (!datelist->data)
@@ -2864,6 +2976,7 @@ void write_toplevel_indices(int amountmsgs)
 	for (j = 0; j <= ATTACHMENT_INDEX; ++j) {
 	    if (!show_index[1][j])
 		continue;
+	    set_dateformat = saved_set_dateformat;
 	    switch (j) {
 		case DATE_INDEX:
 		    writedates(sd->count, sd->first_email);
@@ -2896,23 +3009,38 @@ void write_toplevel_indices(int amountmsgs)
 	        continue;
 
 	    if (!started_line) {
-	      strcpy (period_name, getdatestr (sd->first_email->fromdate));
-	      end_date = getdatestr (sd->last_email->fromdate);
-	      if (strcmp (period_name, end_date)) {
-		strcat (period_name, lang[MSG_TO]);
-		strcat (period_name, end_date);
+	      time_t first_date = sd->first_email->fromdate;
+	      time_t last_date = sd->last_email->fromdate;
+	      if (set_use_sender_date) {
+		  first_date = sd->first_email->date;
+		  last_date = sd->last_email->date;
 	      }
-	      fprintf(fp, "<tr%s><td align=\"left\">%s",
+	      set_dateformat = verbose_dateformat;
+	      strcpy (verbose_period_name, getdatestr (first_date));
+	      set_dateformat = abbr_dateformat;
+	      strcpy (abbr_period_name, getdatestr (first_date));
+	      end_date = getdatestr (last_date);
+	      if (strcmp (abbr_period_name, end_date)) {
+		strcat (abbr_period_name, lang[MSG_TO]);
+		strcat (abbr_period_name, end_date);
+		/* do the same thing for the verbose one */
+		set_dateformat = verbose_dateformat;
+		end_date = getdatestr (last_date);
+		strcat (verbose_period_name, lang[MSG_TO]);
+		strcat (verbose_period_name, end_date);
+	      }
+	      fprintf(fp, "<tr%s><th scope=\"row\" align=\"left\">%s",
 		      (first) ? " class=\"first\"" : "",
 		      (first) ? "<a name=\"first\" id=\"first\"></a>" : "");
 	      /* only add a link to the index if it is not empty */
 	      if (sd->count > 0)
 		fprintf (fp, "<a title=\"%s %s\" href=\"%s%s\">",
-			 period_name, index_title, sd->subdir, index_name[1][j]);
-	      fprintf (fp, "%s", period_name);
+			 verbose_period_name, index_title, 
+			 sd->subdir, index_name[1][j]);
+	      fprintf (fp, "%s", abbr_period_name);
 	      if (sd->count > 0)
 		fprintf (fp, "</a>");
-	      fprintf (fp, "</td>");
+	      fprintf (fp, "</th>");
 	      if (first)
 		first = FALSE;
 	      started_line = 1;
@@ -2922,7 +3050,8 @@ void write_toplevel_indices(int amountmsgs)
 	      /* only add a link to the index if it is not empty */
 	      if (sd->count > 0)
 		fprintf (fp, "<a title=\"%s %s\" href=\"%s%s\">",
-			 period_name, index_title, sd->subdir, index_name[1][j]);
+			 verbose_period_name, index_title, 
+			 sd->subdir, index_name[1][j]);
 	      fprintf (fp, "%s", indextypename[j]);
 	      if (sd->count > 0)
 		fprintf (fp, "</a>");
