@@ -1,5 +1,5 @@
 /*
-** $Id: setup.c,v 1.46 2003/06/30 14:38:34 jose Exp $
+** $Id: setup.c,v 1.54 2004/02/17 18:48:20 pcm Exp $
 */
 
 #include "hypermail.h"
@@ -66,6 +66,10 @@ bool set_nonsequential;
 bool set_warn_surpressions;
 bool set_files_by_thread;
 bool set_href_detection;
+bool set_mbox_shortened;
+bool set_report_new_file;
+bool set_report_new_folder;
+bool set_use_sender_date;
 
 int set_showhtml;
 int set_thrdlevels;
@@ -97,6 +101,7 @@ char *set_mcss_url;
 char *set_label;
 
 char *set_dateformat;
+char *set_indexdateformat;
 char *set_stripsubject;
 
 char *set_link_to_replies;
@@ -219,8 +224,15 @@ struct Config cfg[] = {
      "# cause an HTML-part to be stored as a separate file even if this\n"
      "# option is On.\n", FALSE},
 
-    {"increment", &set_increment, BFALSE, CFG_SWITCH,
-     "# Set this to On to append the input to existing archive.\n", FALSE},
+    {"increment", &set_increment, BFALSE, CFG_INTEGER,
+     "# Set this to -1 to have hypermail figure out whether the input\n"
+     "# is entirely new messages to be appended or whether it contains\n"
+     "# messages that are already in the archive. A value of -1 cannot be\n"
+     "# used with the mbox_shortened option or with the -i command line\n"
+     "# option or with mbox = NONE. Set this to 0 to always treat\n"
+     "# the input as an mbox that contains messages in the archive if the\n"
+     "# existing archive is not empty.\n"
+     "# Set this to 1 to append the input to existing archive.\n", FALSE},
 
     {"readone", &set_readone, BFALSE, CFG_SWITCH,
      "# Set this to On to specify there is only one message in the input.\n", FALSE},
@@ -491,6 +503,10 @@ struct Config cfg[] = {
     {"dateformat", &set_dateformat, NULL, CFG_STRING,
      "# Format (see strftime(3)) for displaying dates.\n", FALSE},
 
+    {"indexdateformat", &set_indexdateformat, NULL, CFG_STRING,
+     "# Format (see strftime(3)) for displaying dates in the index pages.\n"
+     "# Will use dateformat if not specified.\n", FALSE},
+
     {"stripsubject", &set_stripsubject, NULL, CFG_STRING,
      "# A word to be stripped from all subject lines.  Helps unclutter\n"
      "# mailing lists which add tags to subject lines.\n", FALSE},
@@ -502,7 +518,7 @@ struct Config cfg[] = {
      "# %d for the directory name only\n"
      "# %n for the message number\n" "# %c for the content type string\n", FALSE},
 
-    {"spamprotect", &set_spamprotect, BFALSE, CFG_SWITCH,
+    {"spamprotect", &set_spamprotect, BTRUE, CFG_SWITCH,
      "# Set this to On to make hypermail not output real email addresses\n"
      "# in the output HTML but instead it will obfuscate them a little.\n"
      "# You can control the obfuscation with set_antispamdomain.\n", FALSE},
@@ -514,7 +530,7 @@ struct Config cfg[] = {
      "# with some string to confuse screen-scraping programs.\n"
      "# It is probably wise to make this an invalid mail domain.\n", FALSE}, 
 
-    {"spamprotect_id", &set_spamprotect_id, BFALSE, CFG_SWITCH,
+    {"spamprotect_id", &set_spamprotect_id, BTRUE, CFG_SWITCH,
      "# Set this to On to make hypermail not output real email message\n"
      "# ids in HTML comments (sometimes used internally by hypermail) but\n"
      "# instead it will obfuscate them a little so they don't look like\n"
@@ -582,7 +598,7 @@ struct Config cfg[] = {
     {"startmsgnum", &set_startmsgnum, INT(0), CFG_INTEGER,
      "# Sets the number of the first message of an archive. This option is\n"
      "# only active when adding new messages to brand new archive.\n"
-     "# If not set, the default number will be 0000.\n"\
+     "# If not set, the default number will be 0000.\n"
      "# Note that if you change this setting, you are stuck with it. If you\n"
      "# rebuild your archive, you must use the same value or you'll break any\n"
      "# link pointing to your archive.\n", FALSE},
@@ -620,7 +636,9 @@ struct Config cfg[] = {
      "# If folder_by_date or msgsperfolder are in use, create\n"
      "# a symbolic link by this name to the most recently created\n"
      "# subdirectory. Note that many web servers are configured to\n"
-     "# not follow symbolic links for security reasons.\n", FALSE},
+     "# not follow symbolic links for security reasons. The link will\n"
+     "# be created in the directory specified by the 'dir' or '-d' option.",
+     FALSE},
 
     {"base_url", &set_base_url, NULL, CFG_STRING,
      "# The url of the archive's main directory. This is needed when\n"
@@ -724,9 +742,38 @@ struct Config cfg[] = {
 
     {"href_detection", &set_href_detection, BTRUE, CFG_SWITCH,
      "# Set this to On to assume that any string on the body of the message\n"
-     "# that says <A HREF=\"     </A> is a URL, together with its markup\n"
-     "and treat it as such.", TRUE},
+     "# that says <A HREF=\" ... </A> is a URL, together with its markup\n"
+     "# and treat it as such.", TRUE},
 
+    {"mbox_shortened", &set_mbox_shortened, BFALSE, CFG_SWITCH,
+     "# Set this to On to enable use of mbox that has had some of its\n"
+     "# initial messages deleted. Requires usegdbm = 1 and increment = 0.\n"
+     "# The first message in the shortened mbox must have a Message-Id header.\n"
+     "# If discard_dup_msgids is 0, the first message in the shortened mbox\n"
+     "# may not have the same Message-Id as a message that was deleted.\n"
+     "# The mbox may not be altered in any way other than deleting from\n"
+     "# beginning of the mbox or appending new messages to the end (unless\n"
+     "# you rebuild the archive from scratch using a complete mbox).\n", FALSE},
+
+    {"report_new_folder", &set_report_new_folder, BFALSE, CFG_SWITCH,
+     "# Set this to On to have it print (on stdout) the names of any\n"
+     "# new directories created pursuant to the folder_by_date or\n"
+     "# msgsperfolder option, or the initial creation of the archive.\n"
+     "# It will print the full path if that is what you use to specify\n"
+     "# the archive directory. Does not print anything when attachment\n"
+     "# or metadata directories are created.\n", FALSE},
+
+    {"report_new_file", &set_report_new_file, BFALSE, CFG_SWITCH,
+     "# Set this to On to have it print (on stdout) the names of any\n"
+     "# new files created for new messages. It will print the full path\n"
+     "# if that is what you use to specify the archive directory.\n", FALSE},
+
+    {"use_sender_date", &set_use_sender_date, BFALSE, CFG_SWITCH,
+     "# Set this to On to have it use the Date: header (created by the\n"
+     "# the system that sent the message) rather than the date/time the\n"
+     "# message was received, for purposes such as putting in folders\n"
+     "# or sorting. Details of which purposes this affects may change\n"
+     "# in the future.\n", FALSE},
 };
 
 /* ---------------------------------------------------------------- */
