@@ -30,11 +30,14 @@ int rbs_bigtime = 0;
 #include "struct.h"
 #include "parse.h"
 
+#define HAVE_PCRE
+#ifdef HAVE_PCRE
 #ifdef __LCC__
 #include "../lcc/pcre.h"
 #else
 #include <pcre.h>
 #endif
+#endif
 
 struct body *hashnumlookup(int, struct emailinfo **);
 
@@ -55,6 +58,17 @@ unsigned hash(char *s)
     return (hashval % HASHSIZE);
 }
 
+void reinit_structs()
+{
+    int i;
+    for (i = 0; i < HASHSIZE; ++i) {
+	if (etable[i]) {
+	    free(etable[i]);
+	    etable[i] = NULL;
+	}
+    }
+}
+
 void fill_email_dates(struct emailinfo *e, char *date, char *fromdate, char *isodate, char *isofromdate)
 {
     bool fromdate_valid = 1, date_valid = 1;
@@ -123,7 +137,7 @@ void fill_email_dates(struct emailinfo *e, char *date, char *fromdate, char *iso
 		    date_parsed[24] = '\0';
 		    fromdate_parsed[24] = '\0';
 		    rbs_bigtime++;
-					fprintf(stderr, "%d [%d:%d]: %s: received before sent\n %-38.38s %-30.38s\n %-38.38s %-38.38s\n", num, rbs, rbs_bigtime, msgid, fromdate, date, fromdate_parsed, date_parsed);
+		    fprintf(stderr, "%d [%d:%d]: %s: received before sent\n %-38.38s %-30.38s\n %-38.38s %-38.38s\n", num, rbs, rbs_bigtime, msgid, fromdate, date, fromdate_parsed, date_parsed);
 		}
 	    }
 #endif
@@ -190,7 +204,7 @@ struct emailinfo *addhash(int num, char *date, char *name, char *email, char *ms
 	newmsgid = emalloc(MSGDSTRLEN);
 	do {
 	    msgid_dup = 0;
-			sprintf(newmsgid, "%d.%4.4d@hypermail.dummy", time(NULL), freedummy);
+	    sprintf(newmsgid, "%d.%4.4d@hypermail.dummy", time(NULL), freedummy);
 	    h = (struct hashemail *)etable[hash(newmsgid)];
 	    while (h) {
 		if (!strcmp(h->data->msgid, newmsgid)) {
@@ -237,8 +251,9 @@ struct emailinfo *addhash(int num, char *date, char *name, char *email, char *ms
 	e->name = strsav(name);
 
     fill_email_dates(e, date, fromdate, isodate, isofromdate);
-    e->subdir = msg_subdir(e->msgnum, e->date);
-    if (e->subdir) {
+    e->subdir = msg_subdir(e->msgnum, set_use_sender_date ? e->date
+			   : e->fromdate);
+    if (e->subdir && set_increment != -1) {
 	if (!e->subdir->first_email)
 	    e->subdir->first_email = e;
 	e->subdir->last_email = e;
@@ -251,6 +266,7 @@ struct emailinfo *addhash(int num, char *date, char *name, char *email, char *ms
     e->charset = strsav(charset);
     e->flags = 0;
     e->is_deleted = 0;
+    e->deletion_completed = -1;
     e->exp_time = -1;
     e->bodylist = sp;
     e->initial_next_in_thread = -1;
@@ -594,6 +610,43 @@ struct body *hashnumlookup(int num, struct emailinfo **emailp)
     return NULL;
 }
 
+/*
+ * returns info about the first message associated with the given msgid.
+ */
+
+struct emailinfo *hashlookupbymsgid(char *msgid)
+{
+    struct hashemail *ep;
+    if (!msgid || !*msgid)
+	return NULL;
+    ep = etable[hash(msgid)];
+    while (ep) {
+	if (strcmp(msgid, ep->data->msgid) == 0) {
+	    return ep->data;
+	}
+	ep = ep->next;
+    }
+
+    return NULL;
+}
+
+int insert_older_msgs(int num)
+{
+    int i;
+    int num_added = 0;
+    for(i = set_startmsgnum; i < num; ++i) {
+	struct emailinfo *emp;
+	if (hashnumlookup(i, &emp)) {
+	    authorlist = addheader(authorlist, emp, 1, 0);
+	    subjectlist = addheader(subjectlist, emp, 0, 0);
+	    datelist = addheader(datelist, emp, 2, 0);
+	    ++num_added;
+	}
+    }
+    return num_added;
+}
+
+
 /*
  * Find the nearest non-deleted email to num by adding direction to num.
  */
@@ -799,9 +852,9 @@ struct reply *addreply(struct reply *rp, int fromnum, struct emailinfo *email, i
     }
     else {
 #ifdef FASTREPLYCODE
-		for (tempnode = (last_node ? *last_node : rp); tempnode->next != NULL; tempnode = tempnode->next);
-	tempnode->next = newnode;
-	if (rp == threadlist && threadlist_by_msgnum && email)
+      for (tempnode = (last_node ? *last_node : rp); tempnode->next != NULL; tempnode = tempnode->next);
+      tempnode->next = newnode;
+      if (rp == threadlist && threadlist_by_msgnum && email)
 	    threadlist_by_msgnum[email->msgnum] = tempnode;
 #else
 		for (tempnode = rp; tempnode->next != NULL; tempnode = tempnode->next);
@@ -825,7 +878,7 @@ struct reply *addreply2(struct reply *rp, struct emailinfo *from_email, struct e
 	    return rp;		/* don't add 2nd time */
 	}
     }
-	from_email->replylist = addreply(from_email->replylist, from_email->msgnum, email, maybereply, NULL);
+    from_email->replylist = addreply(from_email->replylist, from_email->msgnum, email, maybereply, NULL);
 #endif
     return addreply(rp, from_email->msgnum, email, maybereply, last_node);
 }
@@ -887,6 +940,8 @@ struct header *addheader(struct header *hp, struct emailinfo *email, int sorttyp
 
 	if (sorttype == 2) {
 	    yearsecs = hp->data->datenum = email->fromdate;
+	    if (set_use_sender_date)
+	        yearsecs = hp->data->datenum = email->date;
 	    if (!firstdatenum || yearsecs < firstdatenum)
 		firstdatenum = yearsecs;
 	    if (yearsecs > lastdatenum)
@@ -969,7 +1024,7 @@ struct emailsubdir *new_subdir(char *subdir, struct emailsubdir *last_subdir, ch
     if (insert_point == NULL)
 	insert_point = sd;
     if (insert_point == NULL)
-	folders = new_sd;
+	folders = (set_increment == -1 ? NULL : new_sd);
     else if (date < folders->a_date) {
 	new_sd->next_subdir = folders;
 	folders->prior_subdir = new_sd;
